@@ -24,6 +24,7 @@
 #include <ql/termstructures/credit/defaultprobabilityhelpers.hpp>
 #include <ql/termstructures/credit/flathazardrate.hpp>
 #include <ql/termstructures/yield/flatforward.hpp>
+#include <ql/termstructures/yield/discountcurve.hpp>
 #include <ql/instruments/creditdefaultswap.hpp>
 #include <ql/pricingengines/credit/midpointcdsengine.hpp>
 #include <ql/math/interpolations/linearinterpolation.hpp>
@@ -31,13 +32,23 @@
 #include <ql/math/interpolations/loginterpolation.hpp>
 #include <ql/quotes/simplequote.hpp>
 #include <ql/time/calendars/target.hpp>
+#include <ql/time/calendars/weekendsonly.hpp>
 #include <ql/time/daycounters/actual360.hpp>
 #include <ql/time/daycounters/thirty360.hpp>
 #include <ql/utilities/dataformatters.hpp>
 #include <iomanip>
+#include <map>
+#include <string>
+#include <vector>
+#include <boost/assign/list_of.hpp>
 
 using namespace QuantLib;
 using namespace boost::unit_test_framework;
+using boost::assign::list_of;
+using boost::assign::map_list_of;
+using std::map;
+using std::vector;
+using std::string;
 
 void DefaultProbabilityCurveTest::testDefaultProbability() {
 
@@ -45,7 +56,7 @@ void DefaultProbabilityCurveTest::testDefaultProbability() {
 
     Real hazardRate = 0.0100;
     Handle<Quote> hazardRateQuote = Handle<Quote>(
-                boost::shared_ptr<Quote>(new SimpleQuote(hazardRate)));
+                ext::shared_ptr<Quote>(new SimpleQuote(hazardRate)));
     DayCounter dayCounter = Actual360();
     Calendar calendar = TARGET();
     Size n = 20;
@@ -109,7 +120,7 @@ void DefaultProbabilityCurveTest::testFlatHazardRate() {
 
     Real hazardRate = 0.0100;
     Handle<Quote> hazardRateQuote = Handle<Quote>(
-                boost::shared_ptr<Quote>(new SimpleQuote(hazardRate)));
+                ext::shared_ptr<Quote>(new SimpleQuote(hazardRate)));
     DayCounter dayCounter = Actual360();
     Calendar calendar = TARGET();
     Size n = 20;
@@ -167,14 +178,14 @@ namespace {
         Real recoveryRate = 0.4;
 
         RelinkableHandle<YieldTermStructure> discountCurve;
-        discountCurve.linkTo(boost::shared_ptr<YieldTermStructure>(
+        discountCurve.linkTo(ext::shared_ptr<YieldTermStructure>(
                                     new FlatForward(today,0.06,Actual360())));
 
-        std::vector<boost::shared_ptr<DefaultProbabilityHelper> > helpers;
+        std::vector<ext::shared_ptr<DefaultProbabilityHelper> > helpers;
 
         for(Size i=0; i<n.size(); i++)
             helpers.push_back(
-                boost::shared_ptr<DefaultProbabilityHelper>(
+                ext::shared_ptr<DefaultProbabilityHelper>(
                     new SpreadCdsHelper(quote[i], Period(n[i], Years),
                                         settlementDays, calendar,
                                         frequency, convention, rule,
@@ -183,7 +194,7 @@ namespace {
 
         RelinkableHandle<DefaultProbabilityTermStructure> piecewiseCurve;
         piecewiseCurve.linkTo(
-            boost::shared_ptr<DefaultProbabilityTermStructure>(
+            ext::shared_ptr<DefaultProbabilityTermStructure>(
                 new PiecewiseDefaultCurve<T,I>(today, helpers,
                                                Thirty360())));
 
@@ -205,7 +216,7 @@ namespace {
             CreditDefaultSwap cds(Protection::Buyer, notional, quote[i],
                                   schedule, convention, dayCounter,
                                   true, true, protectionStart);
-            cds.setPricingEngine(boost::shared_ptr<PricingEngine>(
+            cds.setPricingEngine(ext::shared_ptr<PricingEngine>(
                            new MidPointCdsEngine(piecewiseCurve, recoveryRate,
                                                  discountCurve)));
 
@@ -253,14 +264,14 @@ namespace {
         Integer upfrontSettlementDays = 3;
 
         RelinkableHandle<YieldTermStructure> discountCurve;
-        discountCurve.linkTo(boost::shared_ptr<YieldTermStructure>(
+        discountCurve.linkTo(ext::shared_ptr<YieldTermStructure>(
                                     new FlatForward(today,0.06,Actual360())));
 
-        std::vector<boost::shared_ptr<DefaultProbabilityHelper> > helpers;
+        std::vector<ext::shared_ptr<DefaultProbabilityHelper> > helpers;
 
         for(Size i=0; i<n.size(); i++)
             helpers.push_back(
-                boost::shared_ptr<DefaultProbabilityHelper>(
+                ext::shared_ptr<DefaultProbabilityHelper>(
                     new UpfrontCdsHelper(quote[i], fixedRate,
                                          Period(n[i], Years),
                                          settlementDays, calendar,
@@ -272,7 +283,7 @@ namespace {
 
         RelinkableHandle<DefaultProbabilityTermStructure> piecewiseCurve;
         piecewiseCurve.linkTo(
-            boost::shared_ptr<DefaultProbabilityTermStructure>(
+            ext::shared_ptr<DefaultProbabilityTermStructure>(
                 new PiecewiseDefaultCurve<T,I>(today, helpers,
                                                Thirty360())));
 
@@ -300,10 +311,10 @@ namespace {
                                   schedule, convention, dayCounter,
                                   true, true, protectionStart,
                                   upfrontDate,
-                                  boost::shared_ptr<Claim>(),
+                                  ext::shared_ptr<Claim>(),
                                   Actual360(true),
                                   true);
-            cds.setPricingEngine(boost::shared_ptr<PricingEngine>(
+            cds.setPricingEngine(ext::shared_ptr<PricingEngine>(
                            new MidPointCdsEngine(piecewiseCurve, recoveryRate,
                                                  discountCurve, true)));
 
@@ -320,6 +331,25 @@ namespace {
         }
     }
 
+    // Used to check that the exception message contains the expected message string, expMsg.
+    struct ExpErrorPred {
+
+        explicit ExpErrorPred(const string& msg) : expMsg(msg) {}
+
+        bool operator()(const Error& ex) const {
+            string errMsg(ex.what());
+            if (errMsg.find(expMsg) == string::npos) {
+                BOOST_TEST_MESSAGE("Error expected to contain: '" << expMsg << "'.");
+                BOOST_TEST_MESSAGE("Actual error is: '" << errMsg << "'.");
+                return false;
+            } else {
+                return true;
+            }
+        }
+
+        string expMsg;
+    };
+
 }
 
 void DefaultProbabilityCurveTest::testFlatHazardConsistency() {
@@ -365,12 +395,12 @@ void DefaultProbabilityCurveTest::testSingleInstrumentBootstrap() {
     Real recoveryRate = 0.4;
 
     RelinkableHandle<YieldTermStructure> discountCurve;
-    discountCurve.linkTo(boost::shared_ptr<YieldTermStructure>(
+    discountCurve.linkTo(ext::shared_ptr<YieldTermStructure>(
                                     new FlatForward(today,0.06,Actual360())));
 
-    std::vector<boost::shared_ptr<DefaultProbabilityHelper> > helpers(1);
+    std::vector<ext::shared_ptr<DefaultProbabilityHelper> > helpers(1);
 
-    helpers[0] = boost::shared_ptr<DefaultProbabilityHelper>(
+    helpers[0] = ext::shared_ptr<DefaultProbabilityHelper>(
                         new SpreadCdsHelper(quote, tenor,
                                             settlementDays, calendar,
                                             frequency, convention, rule,
@@ -397,6 +427,136 @@ void DefaultProbabilityCurveTest::testUpfrontBootstrap() {
         BOOST_ERROR("Cash-flow settings improperly modified");
 }
 
+/* This test attempts to build a default curve from CDS spreads as of 1 Apr 2020. The spreads are real and from a 
+   distressed reference entity with an inverted CDS spread curve. Using the default IterativeBootstrap with no 
+   retries, the default curve building fails. Allowing retries, it expands the min survival probability bounds but 
+   still fails. We set dontThrow to true in IterativeBootstrap to use a fall back curve.
+*/
+void DefaultProbabilityCurveTest::testIterativeBootstrapRetries() {
+
+    BOOST_TEST_MESSAGE("Testing iterative bootstrap with retries...");
+
+    SavedSettings backup;
+
+    Date asof(1, Apr, 2020);
+    Settings::instance().evaluationDate() = asof;
+    Actual365Fixed tsDayCounter;
+
+    // USD discount curve built out of FedFunds OIS swaps.
+    vector<Date> usdCurveDates = list_of
+        (Date(1, Apr, 2020))
+        (Date(2, Apr, 2020))
+        (Date(14, Apr, 2020))
+        (Date(21, Apr, 2020))
+        (Date(28, Apr, 2020))
+        (Date(6, May, 2020))
+        (Date(5, Jun, 2020))
+        (Date(7, Jul, 2020))
+        (Date(5, Aug, 2020))
+        (Date(8, Sep, 2020))
+        (Date(7, Oct, 2020))
+        (Date(5, Nov, 2020))
+        (Date(7, Dec, 2020))
+        (Date(6, Jan, 2021))
+        (Date(5, Feb, 2021))
+        (Date(5, Mar, 2021))
+        (Date(7, Apr, 2021))
+        (Date(4, Apr, 2022))
+        (Date(3, Apr, 2023))
+        (Date(3, Apr, 2024))
+        (Date(3, Apr, 2025))
+        (Date(5, Apr, 2027))
+        (Date(3, Apr, 2030))
+        (Date(3, Apr, 2035))
+        (Date(3, Apr, 2040))
+        (Date(4, Apr, 2050));
+
+    vector<DiscountFactor> usdCurveDfs = list_of
+        (1.000000000)
+        (0.999955835)
+        (0.999931070)
+        (0.999914629)
+        (0.999902799)
+        (0.999887990)
+        (0.999825782)
+        (0.999764392)
+        (0.999709076)
+        (0.999647785)
+        (0.999594638)
+        (0.999536198)
+        (0.999483093)
+        (0.999419291)
+        (0.999379417)
+        (0.999324981)
+        (0.999262356)
+        (0.999575101)
+        (0.996135441)
+        (0.995228348)
+        (0.989366687)
+        (0.979271200)
+        (0.961150726)
+        (0.926265361)
+        (0.891640651)
+        (0.839314063);
+
+    Handle<YieldTermStructure> usdYts(ext::make_shared<InterpolatedDiscountCurve<LogLinear> >(
+        usdCurveDates, usdCurveDfs, tsDayCounter));
+
+    // CDS spreads
+    map<Period, Rate> cdsSpreads = map_list_of
+        (6 * Months, 2.957980250)
+        (1 * Years, 3.076933100)
+        (2 * Years, 2.944524520)
+        (3 * Years, 2.844498960)
+        (4 * Years, 2.769234420)
+        (5 * Years, 2.713474100);
+    Real recoveryRate = 0.035;
+
+    // Conventions
+    Integer settlementDays = 1;
+    WeekendsOnly calendar;
+    Frequency frequency = Quarterly;
+    BusinessDayConvention paymentConvention = Following;
+    DateGeneration::Rule rule = DateGeneration::CDS2015;
+    Actual360 dayCounter;
+    Actual360 lastPeriodDayCounter(true);
+
+    // Create the CDS spread helpers.
+    vector<ext::shared_ptr<DefaultProbabilityHelper> > instruments;
+    for (map<Period, Rate>::const_iterator it = cdsSpreads.begin(); it != cdsSpreads.end(); ++it) {
+        instruments.push_back(ext::shared_ptr<SpreadCdsHelper>(
+            new SpreadCdsHelper(it->second, it->first, settlementDays, calendar,
+                                frequency, paymentConvention, rule, dayCounter, recoveryRate, usdYts, true, true, Date(),
+                                lastPeriodDayCounter)));
+    }
+
+    // Create the default curve with the default IterativeBootstrap.
+    typedef PiecewiseDefaultCurve<SurvivalProbability, LogLinear, IterativeBootstrap> SPCurve;
+    ext::shared_ptr<DefaultProbabilityTermStructure> dpts = ext::make_shared<SPCurve>(asof, instruments, tsDayCounter);
+
+    // Check that the default curve throws by requesting a default probability.
+    Date testDate(21, Dec, 2020);
+    BOOST_CHECK_EXCEPTION(dpts->survivalProbability(testDate), Error,
+        ExpErrorPred("1st iteration: failed at 1st alive instrument"));
+
+    // Create the default curve with an IterativeBootstrap allowing for 4 retries.
+    // Use a maxFactor value of 1.0 so that we still use the previous survival probability at each pillar. In other
+    // words, the survival probability cannot increase with time so best max at current pillar is the previous 
+    // pillar's value - there is no point increasing it on a retry.
+    IterativeBootstrap<SPCurve> ib(Null<Real>(), Null<Real>(), Null<Real>(), 5, 1.0, 10.0);
+    dpts = ext::make_shared<SPCurve>(asof, instruments, tsDayCounter, ib);
+
+    // Check that the default curve still throws. It throws at the third pillar because the survival probability is 
+    // too low at the second pillar.
+    BOOST_CHECK_EXCEPTION(dpts->survivalProbability(testDate), Error,
+        ExpErrorPred("1st iteration: failed at 3rd alive instrument"));
+
+    // Create the default curve with an IterativeBootstrap that allows for 4 retries and does not throw.
+    IterativeBootstrap<SPCurve> ibNoThrow(Null<Real>(), Null<Real>(), Null<Real>(), 5, 1.0, 10.0, true, 2);
+    dpts = ext::make_shared<SPCurve>(asof, instruments, tsDayCounter, ibNoThrow);
+    BOOST_CHECK_NO_THROW(dpts->survivalProbability(testDate));
+}
+
 
 test_suite* DefaultProbabilityCurveTest::suite() {
     test_suite* suite = BOOST_TEST_SUITE("Default-probability curve tests");
@@ -416,5 +576,7 @@ test_suite* DefaultProbabilityCurveTest::suite() {
                 &DefaultProbabilityCurveTest::testSingleInstrumentBootstrap));
     suite->add(QUANTLIB_TEST_CASE(
                          &DefaultProbabilityCurveTest::testUpfrontBootstrap));
+    suite->add(QUANTLIB_TEST_CASE(
+                &DefaultProbabilityCurveTest::testIterativeBootstrapRetries));
     return suite;
 }
