@@ -29,7 +29,7 @@
 #include <boost/test/unit_test.hpp>
 #endif
 
-#include <boost/timer.hpp>
+#include <boost/timer/timer.hpp>
 
 /* Use BOOST_MSVC instead of _MSC_VER since some other vendors (Metrowerks,
    for example) also #define _MSC_VER
@@ -41,6 +41,15 @@
 #  define BOOST_LIB_NAME boost_unit_test_framework
 #  include <boost/config/auto_link.hpp>
 #  undef BOOST_LIB_NAME
+#  define BOOST_LIB_NAME boost_timer
+#  include <boost/config/auto_link.hpp>
+#  undef BOOST_LIB_NAME
+#  define BOOST_LIB_NAME boost_chrono
+#  include <boost/config/auto_link.hpp>
+#  undef BOOST_LIB_NAME
+#  define BOOST_LIB_NAME boost_system
+#  include <boost/config/auto_link.hpp>
+#  undef BOOST_LIB_NAME
 #endif
 
 /* uncomment the following lines to unmask floating-point exceptions.
@@ -72,6 +81,7 @@
 #include "brownianbridge.hpp"
 #include "businessdayconventions.hpp"
 #include "calendars.hpp"
+#include "callablebonds.hpp"
 #include "capfloor.hpp"
 #include "capflooredcoupon.hpp"
 #include "cashflows.hpp"
@@ -83,6 +93,7 @@
 #include "cms.hpp"
 #include "cmsspread.hpp"
 #include "commodityunitofmeasure.hpp"
+#include "compiledboostversion.hpp"
 #include "compoundoption.hpp"
 #include "convertiblebonds.hpp"
 #include "covariance.hpp"
@@ -105,9 +116,11 @@
 #include "extensibleoptions.hpp"
 #include "fastfouriertransform.hpp"
 #include "fdheston.hpp"
+#include "fdcir.hpp"
 #include "fdmlinearop.hpp"
 #include "fdcev.hpp"
 #include "fdsabr.hpp"
+#include "fittedbonddiscountcurve.hpp"
 #include "forwardoption.hpp"
 #include "forwardrateagreement.hpp"
 #include "functions.hpp"
@@ -119,6 +132,7 @@
 #include "hestonslvmodel.hpp"
 #include "himalayaoption.hpp"
 #include "hybridhestonhullwhiteprocess.hpp"
+#include "indexes.hpp"
 #include "inflation.hpp"
 #include "inflationcapfloor.hpp"
 #include "inflationcapflooredcoupon.hpp"
@@ -209,16 +223,17 @@ using namespace boost::unit_test_framework;
 
 namespace {
 
-    boost::timer t;
+    boost::timer::cpu_timer t;
 
-    void startTimer() { t.restart(); }
+    void startTimer() { t.start(); }
     void stopTimer() {
-        double seconds = t.elapsed();
+        t.stop();
+        std::cout << "\nTests completed in ";
+        double seconds = t.elapsed().wall * 1e-9;
         int hours = int(seconds/3600);
         seconds -= hours * 3600;
         int minutes = int(seconds/60);
         seconds -= minutes * 60;
-        std::cout << " \nTests completed in ";
         if (hours > 0)
             std::cout << hours << " h ";
         if (hours > 0 || minutes > 0)
@@ -246,7 +261,7 @@ namespace {
 #if defined(QL_ENABLE_SESSIONS)
 namespace QuantLib {
 
-    Integer sessionId() { return 0; }
+    ThreadKey sessionId() { return 0; }
 
 }
 #endif
@@ -391,7 +406,9 @@ test_suite* init_unit_test_suite(int, char* []) {
     test->add(FdHestonTest::suite(speed));
     test->add(FdmLinearOpTest::suite());
     test->add(FdCevTest::suite(speed));
+    test->add(FdCIRTest::suite(speed));
     test->add(FdSabrTest::suite(speed));
+    test->add(FittedBondDiscountCurveTest::suite());
     test->add(ForwardOptionTest::suite());
     test->add(ForwardRateAgreementTest::suite());
     test->add(FunctionsTest::suite());
@@ -401,6 +418,7 @@ test_suite* init_unit_test_suite(int, char* []) {
     test->add(GsrTest::suite());
     test->add(HestonModelTest::suite(speed));
     test->add(HybridHestonHullWhiteProcessTest::suite(speed));
+    test->add(IndexTest::suite());
     test->add(InflationTest::suite());
     test->add(InflationCapFloorTest::suite());
     test->add(InflationCapFlooredCouponTest::suite());
@@ -469,12 +487,14 @@ test_suite* init_unit_test_suite(int, char* []) {
     test->add(BarrierOptionTest::experimental());
     test->add(DoubleBarrierOptionTest::experimental());
     test->add(BlackDeltaCalculatorTest::suite());
+    test->add(CallableBondTest::suite());
     test->add(CatBondTest::suite());
     test->add(CdoTest::suite(speed));
     test->add(CdsOptionTest::suite());
     test->add(ChooserOptionTest::suite());
     test->add(CmsSpreadTest::suite());
     test->add(CommodityUnitOfMeasureTest::suite());
+    test->add(CompiledBoostVersionTest::suite());
     test->add(CompoundOptionTest::suite());
     test->add(ConvertibleBondTest::suite());
     test->add(CreditRiskPlusTest::suite());
